#ifndef RANDOM_HPP_
#define RANDOM_HPP_
#define random_hpp_version "$Revision: 1.2 $"

#include <stdint.h>
#include <time.h>
#include "Assertion.hpp"

const uint64_t RANDOM_MASK48ONES        = ((uint64_t)1 << 48) - 1; 
const uint64_t RANDOM_FAKTOR            = ((uint64_t)0x5DEECE66 << 4) + (uint64_t)0xD;
const uint64_t RANDOM_ADDEND            = (uint64_t)0xB;
const double   RANDOM_2POWER54          = (double)((uint64_t)1 << 53);
const uint32_t RANDOM_2POWER32          = 1 << 31;

/**Implements Random by D. H. Lehmer as described by Donald E. Knuth 
in The Art of Computer Programming, Volume 3: Seminumerical Algorithms, section 3.2.1.
This implementation is used in the Java sdk.*/
class Random
{
    /**The seed that was used to initialise the Random instance.*/
    uint64_t init_seed;
    
    /**The current seed that is used to generated the next pseudo-random number.*/
    uint64_t seed;
    
protected:
    /**Auxiliary function to generate a number of random bits. 
     * @param bits: the number of random bits to create (must be between 1 and 31)
     * @return the random bits (rightmost bits)*/
    inline uint32_t nextBits(int bits);
    
public:
    /**Create an instance without specifying a random seed.*/
    inline          Random();
    
    /**Create an instance with the specified seed.
     * @param s: the seed to be used*/
    inline          Random(uint64_t s);
    
    /**Allows to set the seed for further pseudo random number generations.
     * @param s: the seed to be used*/
    inline void     setSeed(uint64_t s);
    
    /**Get the initial seed that was specified by the user. In case the function setSeed has been called more than once, 
     * only the last specified seed is returned.
     * @return the last seed that was set by the user*/
    inline uint64_t getLastInitSeed();
    
    /**Generate a pseudo random boolean number.
     * @return a equally distributed boolean value.*/
    inline bool     nextBool( );
    
    /**Generate a pseudo random double value. 
     * @return an equally distributed double value in the range [0,1)*/
    inline double   nextDouble( );
    
    /**Generate a pseudo random integer value.
     * @return any integer in the range [0,MAX_INT]*/   
    inline uint32_t nextInt( );
    
    /**Generation of a pseudo random integer value in the range [0,n).
    * @param n: the upper bound for the generated pseudo random number. 
    * @return any equally distributed integer in the range [0,n)*/      
    inline uint32_t nextInt(int32_t n);
    

    /**Lazy generation of a pseudo random integer value in the range [0,n).
     * The random value is generated by a simple multiplication with a pseudo random double value from the range [0,1).
     * This function is additional to the Java SDK.
     * @param n: the upper bound for the generated pseudo random number. 
    * @return any integer in the range [0,n)*/      
    inline uint32_t nextIntLazy(int32_t n); 
    
};


// inline implementations
Random::Random()            :init_seed(time(0)) ,seed(init_seed)    { asrt(seed != 0);}
    
Random::Random(uint64_t s)  :init_seed(s)       ,seed(init_seed)    { asrt(seed != 0);}
    
void        Random::setSeed(uint64_t s)         { init_seed= s; seed = s;}

uint64_t    Random::getLastInitSeed()           { return init_seed;}
    
bool        Random::nextBool( )                 { return nextBits(1);}

uint32_t    Random::nextInt( )                  { return nextBits(32);}

uint32_t    Random::nextIntLazy(int32_t n)      { return (uint32_t)(nextDouble() * n); }    

double      Random::nextDouble( )               
            { return ((double)(((uint64_t)nextBits(26) << 27) + nextBits(27))) / RANDOM_2POWER54;}

uint32_t Random::nextBits(int bits){
    
    seed = (seed * RANDOM_FAKTOR + RANDOM_ADDEND) & RANDOM_MASK48ONES;

    return (uint32_t)(seed >> (48 - bits));
}
    
uint32_t    Random::nextInt(int32_t n){
    
    asrt(n > 0);
    
    // n is a power of 2
    if ((n & -n) == n) return (uint32_t)((n * (uint64_t)nextBits(31)) >> 31);
    
    uint32_t bits, val;
    do{
      bits = nextBits(31);
      val = bits % n;
    }while (bits - val + (n-1) > RANDOM_2POWER32);
    return val;
}
    



#endif /*RANDOM_HPP_*/
